extern crate lwo2ultra;
extern crate getopts;

use lwo2ultra::lwofront;
use lwo2ultra::stream::Stream;
use lwo2ultra::ultraback;
use lwo2ultra::converter;
use std::env;
use std::fs::File;
use std::error::Error;
use std::path::Path;
use std::io::Seek;
use getopts::Options;

fn print_usage() {
	println!( "lwo2ultra 1.3-alpha" );
	println!( "Usage: lwo2ultra inputfile [-o outputfile] [-s scalefactor]" );
}

fn main() {
	// parse command line
	let argv: Vec<String> = env::args().collect();

	let in_file_path_str = match argv.iter().nth( 1 ) {			// TODO
		None => { print_usage(); return },
		Some(s) => Path::new( s )
	};
	let in_file_path = Path::new( &in_file_path_str );
	let in_file_dir = String::from( in_file_path.parent().unwrap().to_str().unwrap_or( "." ) );
	let in_file_name = in_file_path.file_name().unwrap().to_str().unwrap();

	let mut opts = Options::new();
	opts.optopt( "s", "", "mesh scaling factor", "SCALE" );
	opts.optopt( "o", "", "output file name", "NAME" );

	let matches = opts.parse( &argv[1..] ).unwrap();
	let scale: f32 = matches.opt_str( "s" ).unwrap_or( String::from( "1.0" ) ).parse().unwrap();

	// open input
	let mut in_file = match File::open( in_file_path ) {
		Err(_) => { println!( "Couldn't open input file" ); return }
		Ok(f) => f,
	};

	// create output
	let out_file_path_str = matches.opt_str( "o" )
		.unwrap_or( in_file_dir + "/" + in_file_name + ".c" );
	let mut out_file = match File::create( out_file_path_str ) {
		Err(_) => { println!( "Couldn't create output file" ); return }
		Ok(f) => f,
	};

	// process input
	let file_size = in_file.seek( std::io::SeekFrom::End( 0 ) ).unwrap() as usize;
	in_file.seek( std::io::SeekFrom::Start( 0 ) ).unwrap();
	let mut reader = Stream::new( in_file, file_size );
	let mut src_mesh = match lwofront::process( &mut reader ) {
		Err(e) => { println!( "{}", e.description() ); return }
		Ok(obj) => obj
	};
	scale_mesh( &mut src_mesh, scale );

	// convert
	let dst_mesh = match converter::convert( &src_mesh, in_file_path.parent().unwrap().to_str().unwrap() ) {
		Err(e) => { println!( "{}", e.description() ); return }
		Ok(obj) => obj,
	};

	// write to output
	match ultraback::save( &dst_mesh, &mut out_file ) {
		Err(e) => println!( "{}", e.description() ),
		Ok(_) => ()
	}
}

fn scale_mesh( mesh: &mut lwofront::Object, scale: f32 ) {
	for layer in &mut mesh.layers {
		for vertex in layer.points.v.iter_mut() {
			vertex.v[0] *= scale;
			vertex.v[1] *= scale;
			vertex.v[2] *= scale;
		}
	}
}
